'use strict'; // Import

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wait = wait;
exports.isTTY = isTTY;
exports.inspect = inspect;
exports.inspectDiff = inspectDiff;
exports.diff = diff;
exports.compare = compare;
exports.log = log;
exports.logComparison = logComparison;
exports.equal = equal;
exports.deepEqual = deepEqual;
exports.contains = contains;
exports.errorEqual = errorEqual;
exports.returnViaCallback = returnViaCallback;
exports.completeViaCallback = completeViaCallback;
exports.returnErrorViaCallback = returnErrorViaCallback;
exports.throwErrorViaCallback = throwErrorViaCallback;
exports.expectViaCallback = expectViaCallback;
exports.expectErrorViaCallback = expectErrorViaCallback;
exports.expectThrowViaFunction = expectThrowViaFunction;
exports.expectFunctionToThrow = exports.expectErrorViaFunction = void 0;

var _util = _interopRequireDefault(require("util"));

var _assert = _interopRequireDefault(require("assert"));

var _ansicolors = _interopRequireDefault(require("ansicolors"));

var _diff = _interopRequireDefault(require("diff"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Alias for setTimeout with paramaters reversed. */
function wait(delay, fn) {
  return setTimeout(fn, delay);
}
/** Whether or not stdout and stderr are interactive. */


function isTTY() {
  return process.stdout && process.stdout.isTTY === true && process.stderr && process.stderr.isTTY === true;
}
/**
 * Return a stringified version of the value with indentation and colors where applicable.
 * Colors will be applied if the environment supports it (--no-colors not present and TTY).
 */


function inspect(value, opts = {}) {
  // If the terminal supports colours, and the user hasn't specified, then default to a sensible default
  const colors = isTTY() && process.argv.indexOf('--no-colors') === -1;
  const depth = 50; // Inspect and return using our defaults

  return _util.default.inspect(value, _objectSpread({
    colors,
    depth
  }, opts));
}
/** Return a highlighted string of a difference. */


function inspectDiff(diff) {
  let result = '';
  diff.forEach(function (part) {
    let value = part.value;

    if (part.added) {
      value = _ansicolors.default.open.black + _ansicolors.default.bgGreen(value) + _ansicolors.default.open.green;
    } else if (part.removed) {
      value = _ansicolors.default.open.black + _ansicolors.default.bgBrightRed(value) + _ansicolors.default.open.green;
    }

    result += value;
  });
  return _ansicolors.default.green(result);
}
/** Return the difference between the new data and the old data. */


function diff(newData, oldData) {
  if (typeof newData === 'object' && typeof oldData === 'object') {
    return _diff.default.diffJson(oldData, newData);
  } else {
    const a = inspect(oldData, {
      colors: false
    });
    const b = inspect(newData, {
      colors: false
    });
    return _diff.default.diffChars(a, b);
  }
}
/** Return a highlighted comparison between the new data and the old data. */


function compare(newData, oldData) {
  return inspectDiff(diff(newData, oldData));
}
/** Log the inspected values of each of the arguments to stdout */


function log(...args) {
  for (let i = 0; i < args.length; ++i) {
    /* eslint no-console:0 */
    console.log(inspect(args[i]));
  }
}
/** Output a comparison of the failed result to stderr */


function logComparison(actual, expected, error) {
  const lines = ['------------------------------------', 'Comparison Error:', _ansicolors.default.green(error.stack || error.message || error), ''];
  lines.push('Comparison Diff:', compare(actual, expected), '');
  lines.push('Comparison Actual:', inspect(actual), '', 'Comparison Expected:', inspect(expected), '------------------------------------'); // Work for node

  if (process.stderr) {
    process.stderr.write(lines.join('\n') + '\n');
  } // Work for browsers
  else {
      console.log(lines.join('\n'));
    }
}
/** Same as assert.equal in that it performs a strict equals check, but if a failure occurs it will output detailed information */


function equal(actual, expected, testName = 'equal assertion', next) {
  try {
    _assert.default.equal(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Same as assert.deepEQual in that it performs a deep equals check, but if a failure occurs it will output detailed information */


function deepEqual(actual, expected, testName = 'deep equal assertion', next) {
  try {
    _assert.default.deepEqual(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if the actual result contains the expected result .*/


function contains(actual, expected, testName = 'contains assertion', next) {
  if (testName == null) testName = `Expected [${actual}] to contain [${expected}]`;

  try {
    _assert.default.ok(actual.indexOf(expected) !== -1, testName);
  } catch (checkError) {
    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if an error was as expected, if a failure occurs it will output detailed information */


function errorEqual(actualError, expectedError, testName = 'error equal assertion', next) {
  let expectedErrorMessage, actualErrorMessage;

  if (expectedError) {
    if (expectedError instanceof Error) {
      expectedErrorMessage = expectedError.message;
    } else {
      expectedErrorMessage = expectedError;
      expectedError = new Error(expectedErrorMessage);
    }
  }

  if (actualError) {
    if (actualError instanceof Error) {
      actualErrorMessage = actualError.message;
    } else {
      actualErrorMessage = actualError;
      actualError = new Error(actualErrorMessage);
    }
  }

  try {
    if (actualErrorMessage && expectedErrorMessage) {
      contains(actualErrorMessage, expectedErrorMessage, testName);
    } else {
      equal(actualError, expectedError || null, testName);
    }
  } catch (checkError) {
    logComparison(actualError && (actualError.stack || actualError.message || actualError), expectedErrorMessage, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Generate a callback that will return the specified value. */


function returnViaCallback(value) {
  return function () {
    return value;
  };
}
/** Generate a callback that will receive a completion callback and call it with the specified result after the specified delay. */

/* eslint no-magic-numbers:0 */


function completeViaCallback(value, delay = 100) {
  return function (complete) {
    wait(delay, function () {
      complete(null, value);
    });
  };
}
/** Generate a callback that return an error instance with the specified message/error. */


function returnErrorViaCallback(error = 'an error occured') {
  return function () {
    if (error instanceof Error) {
      return error;
    } else {
      return new Error(error);
    }
  };
}
/** Generate a callback that throw an error instance with the specified message/error. */


function throwErrorViaCallback(error = 'an error occured') {
  return function () {
    if (error instanceof Error) {
      throw error;
    } else {
      throw new Error(error);
    }
  };
}
/** Generate a callback that will check the arguments it received with the arguments specified, if a failure occurs it will output detailed information. */


function expectViaCallback(...expected) {
  return (...actual) => deepEqual(actual, expected);
}
/** Generate a callback that will check the error (if any) it receives for the expected error (if any), if a failure occurs it will output detailed information. */


function expectErrorViaCallback(expected, testName = 'expect error via callback assertion', next) {
  return actual => errorEqual(actual, expected, testName, next);
}
/** Expect the passed function to throw an error at some point. */


function expectThrowViaFunction(expected, fn, testName = 'expect error via function assertion', next) {
  let actual = null;

  try {
    fn();
  } catch (error) {
    actual = error;
  }

  errorEqual(actual, expected, testName, next);
}
/** Deprecated. Use {@link expectErrorViaFunction} instead. */


const expectErrorViaFunction = expectThrowViaFunction;
/** Deprecated. Use {@link expectErrorViaFunction} instead. */

exports.expectErrorViaFunction = expectErrorViaFunction;
const expectFunctionToThrow = expectThrowViaFunction;
exports.expectFunctionToThrow = expectFunctionToThrow;